# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Connector, Resource, Pins, Clock
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, ButtonResources, Display7SegResource, UARTResource, SPIFlashResources,
	VGAResource
)


__all__ = (
	'NandlandGoPlatform',
)


class NandlandGoPlatform(ICE40Platform):
	device      = 'iCE40HX1K'
	package     = 'VQ100'
	default_clk = 'clk25'

	resources = [
		Resource(
			'clk25', 0, Pins('15', dir = 'i'), Clock(25e6)
		),

		*LEDResources(pins = '56 57 59 60'),
		*ButtonResources(pins = '53 51 54 52'),

		Display7SegResource(0,
			a = '3', b = '4', c = '93', d = '91', e = '90', f = '1', g = '2',
			invert = True
		),
		Display7SegResource(1,
			a = '100', b = '99', c = '97', d = '95', e = '94', f = '8', g = '96',
			invert = True
		),

		UARTResource(0, rx = '73', tx = '74'),

		*SPIFlashResources(0, cs_n = '49', clk = '48', copi = '45', cipo = '46'),

		VGAResource(0,
			r = '36 37 40',
			g = '29 30 33',
			b = '28 41 42',
			hs = '26', vs = '27'
		),
	]
	connectors = [
		Connector('pmod', 0, '65 64 63 62 - - 78 79 80 81 - -'),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	NandlandGoPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import (
	Connector, Resource, Pins, Clock, Attrs, PinsN,
	Subsignal
)
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, SwitchResources, UARTResource, SPIFlashResources
)

__all__ = (
	'ICESugarPlatform',
)


class ICESugarPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'LVCMOS33')
		),

		*LEDResources(pins = '40 41 39', invert = True, attrs = Attrs(IO_STANDARD = 'LVCMOS33')),

		# Semantic aliases
		Resource('led_r', 0, PinsN('40', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),
		Resource('led_g', 0, PinsN('41', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),
		Resource('led_b', 0, PinsN('39', dir = 'o'), Attrs(IO_STANDARD = 'LVCMOS33')),

		*SwitchResources(
			pins = '18 19 20 21',
			attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),

		UARTResource(0,
			rx = '4', tx = '6',
			attrs = Attrs(IO_STANDARD = 'LVTTL33', PULLUP = 1)
		),

		*SPIFlashResources(0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '12', hold_n = '13',
			attrs = Attrs(IO_STANDARD = 'LVCMOS33')
		),

		Resource('usb', 0,
			Subsignal('d_p',    Pins('10', dir = 'io')),
			Subsignal('d_n',    Pins('9', dir = 'io')),
			Subsignal('pullup', Pins('11', dir = 'o')),
			Attrs(IO_STANDARD = 'LVCMOS33')
		),
	]

	connectors = [
		Connector('pmod', 0, '10  6  3 48 - -  9  4  2 47 - -'), # PMOD1 - IO pins shared by USB
		Connector('pmod', 1, '46 44 42 37 - - 45 43 38 36 - -'), # PMOD2
		Connector('pmod', 2, '34 31 27 25 - - 32 28 26 23 - -'), # PMOD3
		Connector('pmod', 3, '21 20 19 18 - -  -  -  -  - - -'), # PMOD4 - IO pins used for switches via jumpers
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICESugarPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import (
	Connector, Resource, Pins, Clock, Attrs, Subsignal, PinsN
)
from torii.build.run              import BuildProducts
from torii.platform.vendor.altera import AlteraPlatform
from torii.platform.resources     import (
	LEDResources, ButtonResources, SwitchResources, UARTResource, SPIResource,
	SDRAMResource, SDCardResources, VGAResource
)

__all__ = (
	'MisterPlatform',
)


# The MiSTer platform is built around the DE10-Nano; if you update one you should update the other.
class MisterPlatform(AlteraPlatform):
	device      = '5CSEBA6' # Cyclone V 110K LEs
	package     = 'U23'     # UBGA-484
	speed       = 'I7'
	default_clk = 'clk50'

	resources   = [
		Resource(
			'clk50', 0, Pins('V11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('Y13', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('E11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'W15 AA24 V16 V15 AF26 AE26 Y16 AA23',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'AH17 AH16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'Y24 W24 W21 W20',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# Arduino header
		UARTResource(0,
			rx = 'AG13', tx = 'AF13',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# LTC2308 analogue-to-digital converter
		SPIResource(0,
			cs_n = 'U9', clk = 'V10', copi = 'AC4', cipo = 'AD4',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# ADV7513 HDMI transmitter
		# Note this has a lot of input formats for tx_d, but this defaults to RGB24
		Resource('adv7513', 0,
			Subsignal('tx_d_r', Pins('AD12 AE12 W8   Y8   AD11 AD10 AE11 Y5',  dir = 'o')),
			Subsignal('tx_d_g', Pins('AF10 Y4   AE9  AB4  AE7  AF6  AF8  AF5', dir = 'o')),
			Subsignal('tx_d_b', Pins('AE4  AH2  AH4  AH5  AH6  AG6  AF9  AE8', dir = 'o')),
			Subsignal('tx_clk', Pins('AG5',  dir = 'o')),
			Subsignal('tx_de',  Pins('AD19', dir = 'o')),
			Subsignal('tx_hs',  Pins('T8',   dir = 'o')),
			Subsignal('tx_vs',  Pins('V13',  dir = 'o')),
			Subsignal('tx_int', Pins('AF11', dir = 'i')),
			Subsignal('i2s0',   Pins('T13',  dir = 'o')),
			Subsignal('mclk',   Pins('U11',  dir = 'o')),
			Subsignal('lrclk',  Pins('T11',  dir = 'o')),
			Subsignal('sclk',   Pins('T12',  dir = 'o')),
			Subsignal('scl',    Pins('U10',  dir = 'io')),
			Subsignal('sda',    Pins('AA4',  dir = 'io')),
			Attrs(io_standard = '3.3-V LVTTL')
		),

		# MiSTer SDRAM Board (required)
		# https://github.com/MiSTer-devel/Hardware_MiSTer/blob/master/releases/sdram_xs_2.2.pdf
		SDRAMResource(0,
			clk = '20', cs_n = '33', we_n = '27', ras_n = '32', cas_n = '31',
			ba = '34 35', a = '37 38 39 40 28 25 26 23 24 21 36 22 19',
			dq = '1 2 3 4 5 6 7 8 18 17 16 15 14 13 9 10',
			dqm = '', conn = ('gpio', 0), attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),

		# MiSTer I/O Board (optional, but highly recommended)
		# https://github.com/MiSTer-devel/Hardware_MiSTer/blob/master/releases/iobrd_6.0.pdf
		Resource(
			'power_led', 0, PinsN('1', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'disk_led', 0, PinsN('3', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'user_led', 0, PinsN('5', dir = 'o', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),

		Resource(
			'reset_switch', 0, PinsN('17', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'osd_switch', 0, PinsN('13', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'user_switch', 0, PinsN('15', dir = 'i', conn = ('gpio', 1)), Attrs(io_standard = '3.3-V LVTTL')
		),

		Resource('audio', 0,
			Subsignal('l', Pins('2', dir = 'o', conn = ('gpio', 1))),
			Subsignal('r', Pins('7', dir = 'o', conn = ('gpio', 1))),
			Attrs(io_standard = '3.3-V LVTTL')
		),

		Resource('toslink', 0, Pins('9', dir = 'o', conn = ('gpio', 1))),

		*SDCardResources(0,
			clk = '13', cmd = '8',
			dat0 = '16', dat1 = '18', dat2 = '4', dat3 = '6',
			conn = ('gpio', 1), attrs = Attrs(io_standard = '3.3-V LVTTL')
		),

		# The schematic is difficult to understand here...
		VGAResource(0,
			r = '28 32 34 36 38 40',
			g = '27 31 33 35 37 39',
			b = '21 23 25 26 24 24',
			hs = '20', vs = '19',
			conn = ('gpio', 1),
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		)
	]

	connectors  = [
		# Located on the top of the board, above the chip.
		Connector('gpio', 0,
			'V12  E8   W12  D11  D8   AH13 AF7  AH14 AF4  AH3  '
			'-    -    AD5  AG14 AE23 AE6  AD23 AE24 D12  AD20 '
			'C12  AD17 AC23 AC22 Y19  AB23 AA19 W11  -    -    '
			'AA18 W14  Y18  Y17  AB25 AB26 Y11  AA26 AA13 AA11 '
		),
		# Located on the bottom of the board.
		Connector('gpio', 1,
			'Y15  AC24 AA15 AD26 AG28 AF28 AE25 AF27 AG26 AH27 '
			'-    -    AG25 AH26 AH24 AF25 AG23 AF23 AG24 AH22 '
			'AH21 AG21 AH23 AA20 AF22 AE22 AG20 AF21 -    -    '
			'AG19 AH19 AG18 AH18 AF18 AF20 AG15 AE20 AE19 AE17 '
		),
		Connector('arduino', 0,
			'AG13 AF13 AG10 AG9  U14  U13  AG8  AH8  '
			'AF17 AE15 AF15 AG16 AH11 AH12 AH9  AG11 '
			'AH7  -    -    -    -    -    -    -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	MisterPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Connector, Resource, Attrs, PinsN
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import LEDResources, SPIFlashResources

__all__ = (
	'UpduinoV1Platform',
)


class UpduinoV1Platform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'SB_HFOSC'
	hfosc_div   = 0

	resources   = [
		*LEDResources(
			pins = '39 40 41', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource('led_g', 0, PinsN('39', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_b', 0, PinsN('40', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, PinsN('41', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		*SPIFlashResources(0,
			cs_n = '16', clk = '15', cipo = '17', copi = '14',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		# 'Left' row of header pins (JP5 on the schematic)
		Connector('j', 0, '- - 23 25 26 27 32 35 31 37 34 43 36 42 38 28'),
		# 'Right' row of header pins (JP6 on the schematic)
		Connector('j', 1, '12 21 13 19 18 11 9 6 44 4 3 48 45 47 46 2')
	]

	# This board doesn't have an integrated programmer.



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                      import Connector, Resource, Pins, Subsignal
from torii.build.run                  import BuildProducts
from torii.platform.vendor.quicklogic import QuicklogicPlatform
from torii.platform.resources         import (
	ButtonResources, RGBLEDResource, UARTResource, SPIResource, I2CResource,
	DirectUSBResource
)

__all__ = (
	'QuickfeatherPlatform',
)


class QuickfeatherPlatform(QuicklogicPlatform):
	device      = 'ql-eos-s3_wlcsp'
	package     = 'PU64'
	default_clk = 'sys_clk0'
	# It is possible to configure both oscillator frequency and
	# clock divider. Resulting frequency is: 60MHz / 12 = 5MHz
	osc_freq    = int(60e6)
	osc_div     = 12

	connectors = [
		Connector('J', 2, '- 28 22 21 37 36 42 40 7 2 4 5'),
		Connector('J', 3, '- 8 9 17 16 20 6 55 31 25 47 - - - - 41'),
		Connector('J', 8, '27 26 33 32 23 57 56 3 64 62 63 61 59 - - -'),
	]

	resources   = [
		*ButtonResources(pins = '62'),

		RGBLEDResource(0, r = '34', g = '39', b = '38'),

		UARTResource(0,
			rx = '9', tx = '8',
		),

		SPIResource(0,
			cs_n = '11', clk = '20', copi = '16', cipo = '17'
		),
		SPIResource(1,
			cs_n = '37', clk = '40', copi = '36', cipo = '42',
			role = 'peripheral'
		),

		I2CResource(0,
			scl = '4', sda = '5'
		),
		I2CResource(1,
			scl = '22', sda = '21'
		),

		DirectUSBResource(0, d_p = '10', d_n = '14'),

		Resource('swd', 0,
			Subsignal('clk', Pins('54', dir = 'io')),
			Subsignal('io',  Pins('53', dir = 'io')),
		),
	]

	# This programmer requires OpenOCD with support for eos-s3:
	# https://github.com/antmicro/openocd/tree/eos-s3-support
	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}.openocd', '{name}_iomux.openocd') as files:
			(bitstream_openocd_filename, iomux_openocd_filename) = files
			check_call([
				openocd,
				'-s', 'tcl',
				'-f', 'interface/ftdi/antmicro-ftdi-adapter.cfg',
				'-f', 'interface/ftdi/swd-resistor-hack.cfg',
				'-f', 'board/quicklogic_quickfeather.cfg',
				'-f', bitstream_openocd_filename,
				'-c', 'init',
				'-c', 'reset halt',
				'-c', 'load_bitstream',
				'-f', iomux_openocd_filename,
				'-c', 'exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	QuickfeatherPlatform().build(Blinky())



# SPDX-License-Identifier: BSD-2-Clause

from typing                       import Literal, Dict

from torii.build                  import (
	Resource, Pins, Clock, Attrs, Subsignal, Connector, PinsN,
	DiffPairs
)
from torii.build.run              import BuildProducts, BuildPlan
from torii.hdl.ir                 import Fragment
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import (
	ButtonResources, SwitchResources, LEDResources, UARTResource, I2CResource,
	SPIResource, VGAResource, SDCardResources, ULPIResource
)

__all__ = (
	'Genesys2Platform',
)


class Genesys2Platform(XilinxPlatform):
	'''Platform file for Diglient Genesys2 Kitex-7 board.
	https://reference.digilentinc.com/reference/programmable-logic/genesys-2/start'''

	device  = 'xc7k325t'
	package = 'ffg900'
	speed   = '2'

	def __init__(self, JP6: Literal['1V2', '1V8', '2V5', '3V3'] = '2V5') -> None:
		super().__init__()

		if JP6 not in ('1V2', '1V8', '2V5', '3V3'):
			raise ValueError(f'JP6 must be one of \'1V2\', \'1V8\', \'2V5\', \'3V3\', not {JP6!r}')

		self._JP6 = JP6

	def bank15_16_17_iostandard(self) -> str:
		return 'LVCMOS' + self._JP6.replace('V', '')

	default_rst = 'rst'
	default_clk = 'clk'

	resources = [
		Resource('rst', 0, PinsN('R19', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'clk', 0, DiffPairs(p = 'AD12 ', n = 'AD11', dir = 'i'), Clock(200e6),
			Attrs(IOSTANDARD = 'LVDS')
		),
		*ButtonResources(pins = {
				0: 'E18',
				1: 'B19',
				2: 'C19',
				3: 'M19',
				4: 'M20'
			}, attrs = Attrs(IOSTANDARD = bank15_16_17_iostandard)
		),
		*SwitchResources(
			pins = 'G19 G25 H24 K19 N19 P19',
			attrs = Attrs(IOSTANDARD = bank15_16_17_iostandard)
		),
		*SwitchResources(pins = {
				6: 'P26',
				7: 'P27'
			}, attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*LEDResources(
			pins = 'T28 V19 U30 U29 V20 V26 W24 W23',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('fan', 0,
			Subsignal('pwm', Pins('W19', dir = 'o')),
			Subsignal('tach', Pins('V21', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		UARTResource(0,
			rx = 'Y20', tx = 'Y23',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		I2CResource(0,
			scl = 'AE30', sda = 'AF30',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('ddr3', 0,
			Subsignal('rst', PinsN('AG5', dir = 'o'), Attrs(IOSTANDARD = 'SSTL15')),
			Subsignal('clk',
				DiffPairs(p = 'AB9', n = 'AC9', dir = 'o'),
				Attrs(IOSTANDARD = 'DIFF_SSTL15_DCI')
			),
			Subsignal('clk_en', Pins('AJ9', dir = 'o')),
			Subsignal('cs', PinsN('AH12', dir = 'o')),
			Subsignal('we', PinsN('AG13', dir = 'o')),
			Subsignal('ras', PinsN('AE11', dir = 'o')),
			Subsignal('cas', PinsN('AF11', dir = 'o')),
			Subsignal('a', Pins(
				'AC12 AE8 AD8 AC10 AD9 AA13 AA10 AA11 '
				'Y10 Y11 AB8 AA8 AB12 AA12 AH9 AG9', dir = 'o')
			),
			Subsignal('ba', Pins('AE9 AB10 AC11', dir = 'o')),
			Subsignal('dqs',
				DiffPairs(
					p   = 'AD2 AG4 AG2 AH7',
					n   = 'AD1 AG3 AH1 AJ7',
					dir = 'io'
				),
				Attrs(IOSTANDARD = 'DIFF_SSTL15_DCI', ODT = 'RTT_40')
			),
			Subsignal('dq', Pins(
				'AD3 AC2 AC1 AC5 AC4 AD6 AE6 AC7 '
				'AF2 AE1 AF1 AE4 AE3 AE5 AF5 AF6 '
				'AJ4 AH6 AH5 AH2 AJ2 AJ1 AK1 AJ3 '
				'AF7 AG7 AJ6 AK6 AJ8 AK8 AK5 AK4', dir = 'io'),
				Attrs(ODT = 'RTT_40')
			),
			Subsignal('dm', Pins('AD4 AF3 AH4 AF8', dir = 'o')),
			Subsignal('odt', Pins('AK9', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL15_DCI', SLEW = 'FAST', OUTPUT_IMPEDANCE = 'RDRV_40_40')
		),
		Resource('audio_i2c', 0,  # ADAU1761 I2C
			Subsignal('scl', Pins('AE19', dir = 'io')),
			Subsignal('sda', Pins('AF18', dir = 'io')),
			Subsignal('adr', Pins('AD19 AG19', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('audio_i2s', 0,  # ADAU1761 ADC, I2S
			Subsignal('clk', Pins('AG18', dir = 'o')), # BCLK
			Subsignal('sd_adc', Pins('AH19', dir = 'o')), # ADC_SDATA
			Subsignal('sd_dac', Pins('AJ19', dir = 'i')),  # DAC_SDATA
			Subsignal('ws', Pins('AJ18', dir = 'o')), # LRCLK
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('audio_clk', 0,  # ADAU1761 MCLK
			Pins('AK19', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS18')
		),
		SPIResource(0,  # OLED, SSD1306, 128 x 32
			cs_n = 'dummy-cs0', clk = 'AF17', copi = 'Y15',
			cipo = 'dummy-cipo0', reset = 'AB17',
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('oled', 0,  # OLED, UG-2832HSWEG04
			Subsignal('dc', Pins('AC17', dir = 'o')),
			Subsignal('vdd_en', PinsN('AG17', dir = 'o')),
			Subsignal('vbat_en', PinsN('AB22', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('hdmi', 0,  # HDMI TX, connector J4
			Subsignal('scl', Pins('AF27', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('sda', Pins('AF26', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('clk', DiffPairs(p = 'AA20', n = 'AB20', dir = 'o')),
			Subsignal('d', DiffPairs(
				p = 'AC20 AA22 AB24',
				n = 'AC21 AA23 AC25',
				dir = 'o'
			)),
			Attrs(IOSTANDARD = 'TMDS_33')
		),
		Resource('hdmi', 1,  # HDMI RX, connector J5
			Subsignal('scl', Pins('AJ28', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('sda', Pins('AJ29', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('clk', DiffPairs(p = 'AE28', n = 'AF28', dir = 'i')),
			Subsignal('rx', DiffPairs(
				p = 'AJ26 AG27 AH26',
				n = 'AK26 AG28 AH27',
				dir = 'i'
			)),
			Attrs(IOSTANDARD = 'TMDS_33')
		),
		VGAResource(0,
			r = 'AK25 AG25 AH25 AK24 AJ24',
			g = 'AJ23 AJ22 AH22 AK21 AJ21 AK23',
			b = 'AH20 AG20 AF21 AK20 AG22',
			hs = 'AF20', vs = 'AG23', invert_sync = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SDCardResources(0,
			clk = 'R28', cmd = 'R29', dat0 = 'R26', dat1 = 'R30',
			dat2 = 'P29', dat3 = 'T30', cd = 'P28',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('sd_card_rst', 0,
			Pins('AE24', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		ULPIResource(0,
			data = 'AE14 AE15 AC15 AC16 AB15 AA15 AD14 AC14',
			rst = 'AB14', clk = 'AD18', dir = 'Y16', stp = 'AA17', nxt = 'AA16',
			clk_dir = 'i', rst_invert = True, attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('vusb_oc', 0,
			PinsN('AF16', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource('eth_rgmii', 0,
			Subsignal('rst', PinsN('AH24', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('mdc', Pins('AF12', dir = 'o')),
			Subsignal('mdio', Pins('AG12', dir = 'io')),
			Subsignal('tx_clk', Pins('AE10', dir = 'o')),
			Subsignal('tx_ctl', Pins('AK14', dir = 'o')),
			Subsignal('tx_data', Pins('AJ12 AK11 AJ11 AK10', dir = 'o')),
			Subsignal('rx_clk', Pins('AG10', dir = 'i')),
			Subsignal('rx_ctl', Pins('AH11', dir = 'i')),
			Subsignal('rx_data', Pins('AJ14 AH14 AK13 AJ13', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS15')
		)
	]

	connectors = [
		Connector('pmod', 0,  # JA
			'U27 U28 T26 T27 - - '
			'T22 T23 T20 T21 - - '
		),
		Connector('pmod', 1,  # JB
			'V29 V30 V25 W26 - - '
			'T25 U25 U22 U23 - - '
		),
		Connector('pmod', 2,  # JC
			'AC26 AJ27 AH30 AK29 - - '
			'AD26 AG30 AK30 AK28 - - '
		),
		Connector('pmod', 3,  # JD
			'V27 Y30 V24 W22 - - '
			'U24 Y26 V22 W21 - - '
		),
		Connector('pmod', 4,  # JXADC
			'J23 K23 L22 L21 - - '
			'J24 K24 L23 K21 - - '
		),
		Connector('hpc', 0, {
			'dp1_m2c_p': 'Y6',
			'dp1_m2c_n': 'Y5',
			'dp2_m2c_p': 'W4',
			'dp2_m2c_n': 'W3',
			'dp3_m2c_p': 'V6',
			'dp3_m2c_n': 'V5',
			'dp1_c2m_p': 'V2',
			'dp1_c2m_n': 'V1',
			'dp2_c2m_p': 'U4',
			'dp2_c2m_n': 'U3',
			'dp3_c2m_p': 'T2',
			'dp3_c2m_n': 'T1',
			'dp0_c2m_p': 'Y2',
			'dp0_c2m_n': 'Y1',
			'dp0_m2c_p': 'AA4',
			'dp0_m2c_n': 'AA3',
			'la06_p': 'D29',
			'la06_n': 'C30',
			'la10_p': 'B27',
			'la10_n': 'A27',
			'la14_p': 'C24',
			'la14_n': 'B24',
			'la18_cc_p': 'D17',
			'la18_cc_n': 'D18',
			'la27_p': 'A20',
			'la27_n': 'A21',
			'ha01_cc_p': 'M28',
			'ha01_cc_n': 'L28',
			'ha05_p': 'J29',
			'ha05_n': 'H29',
			'ha09_p': 'L30',
			'ha09_n': 'K30',
			'ha13_p': 'K26',
			'ha13_n': 'J26',
			'ha16_p': 'M22',
			'ha16_n': 'M23',
			'ha20_p': 'G27',
			'ha20_n': 'F27',
			'clk1_m2c_p': 'E28',
			'clk1_m2c_n': 'D28',
			'la00_cc_p': 'D27',
			'la00_cc_n': 'C27',
			'la03_p': 'E29',
			'la03_n': 'E30',
			'la08_p': 'C29',
			'la08_n': 'B29',
			'la12_p': 'F26',
			'la12_n': 'E26',
			'la16_p': 'E23',
			'la16_n': 'D23',
			'la20_p': 'G22',
			'la20_n': 'F22',
			'la22_p': 'J17',
			'la22_n': 'H17',
			'la25_p': 'D22',
			'la25_n': 'C22',
			'la29_p': 'B18',
			'la29_n': 'A18',
			'la31_p': 'C17',
			'la31_n': 'B17',
			'la33_p': 'D16',
			'la33_n': 'C16',
			'ha03_p': 'N25',
			'ha03_n': 'N26',
			'ha07_p': 'M29',
			'ha07_n': 'M30',
			'ha11_p': 'P23',
			'ha11_n': 'N24',
			'ha14_p': 'N27',
			'ha14_n': 'M27',
			'ha18_p': 'E19',
			'ha18_n': 'D19',
			'ha22_p': 'D21',
			'ha22_n': 'C21',
			'gbtclk1_m2c_p': 'N8',
			'gbtclk1_m2c_n': 'N7',
			'gbtclk0_m2c_p': 'L8',
			'gbtclk0_m2c_n': 'L7',
			'la01_cc_p': 'D26',
			'la01_cc_n': 'C26',
			'la05_p': 'B30',
			'la05_n': 'A30',
			'la09_p': 'B28',
			'la09_n': 'A28',
			'la13_p': 'E24',
			'la13_n': 'D24',
			'la17_cc_p': 'F21',
			'la17_cc_n': 'E21',
			'la23_p': 'G17',
			'la23_n': 'F17',
			'la26_p': 'B22',
			'la26_n': 'A22',
			'pg_m2c': 'AH21',
			'ha00_cc_p': 'K28',
			'ha00_cc_n': 'K29',
			'ha04_p': 'M24',
			'ha04_n': 'M25',
			'ha08_p': 'J27',
			'ha08_n': 'J28',
			'ha12_p': 'L26',
			'ha12_n': 'L27',
			'ha15_p': 'J21',
			'ha15_n': 'J22',
			'ha19_p': 'G29',
			'ha19_n': 'F30',
			'prsnt_m2c_b': 'AA21',
			'clk0_m2c_p': 'F20',
			'clk0_m2c_n': 'E20',
			'la02_p': 'H30',
			'la02_n': 'G30',
			'la04_p': 'H26',
			'la04_n': 'H27',
			'la07_p': 'F25',
			'la07_n': 'E25',
			'la11_p': 'A25',
			'la11_n': 'A26',
			'la15_p': 'B23',
			'la15_n': 'A23',
			'la19_p': 'H21',
			'la19_n': 'H22',
			'la21_p': 'L17',
			'la21_n': 'L18',
			'la24_p': 'H20',
			'la24_n': 'G20',
			'la28_p': 'J19',
			'la28_n': 'H19',
			'la30_p': 'A16',
			'la30_n': 'A17',
			'la32_p': 'K18',
			'la32_n': 'J18',
			'ha02_p': 'P21',
			'ha02_n': 'P22',
			'ha06_p': 'N29',
			'ha06_n': 'N30',
			'ha10_p': 'N21',
			'ha10_n': 'N22',
			'ha17_cc_p': 'C25',
			'ha17_cc_n': 'B25',
			'ha21_p': 'G28',
			'ha21_n': 'F28',
			'ha23_p': 'G18',
			'ha23_n': 'F18'
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_after_read': 'auto_detect_xpm',
			'script_before_bitstream':
			'set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]',
			'add_constraints': '''
			set_property BITSTREAM.CONFIG.CONFIGRATE 50 [current_design]
			set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]
			set_property BITSTREAM.CONFIG.SPI_32BIT_ADDR YES [current_design]
			set_property BITSTREAM.CONFIG.SPI_FALL_EDGE YES [current_design]
			set_property CFGBVS VCCO [current_design]
			set_property CONFIG_VOLTAGE 3.3 [current_design]
			'''
		}
		return super().toolchain_prepare(
			fragment, name, **overrides, **kwargs
		)

	@property
	def file_templates(self) -> Dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': r'''
			source [find interface/ftdi/digilent-hs1.cfg]
			# fix channel
			ftdi_channel 1
			adapter_khz 25000
			transport select jtag
			source [find cpld/xilinx-xc7.cfg]
			source [find cpld/jtagspi.cfg]
			'''
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.bit') as files:
			(config_filename, bitstream_filename) = files

			check_call([
				openocd,
				'-f', config_filename,
				'-c', f'init; pld load 0 {bitstream_filename}; exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	Genesys2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build     import Resource, Attrs, Clock, Pins
from torii.build.run import BuildProducts

from .upduino_v1     import UpduinoV1Platform

__all__ = (
	'UpduinoV2Platform',
)


class UpduinoV2Platform(UpduinoV1Platform):
	# Mostly identical to the V1 board, but it has an integrated
	# programmer and a 12MHz oscillator which is NC by default.
	resources = UpduinoV1Platform.resources + [
		# Solder pin 12 to the adjacent 'J8' osc_out pin to enable.
		Resource(
			'clk12', 0, Pins('12', dir = 'i'), Clock(12e6), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	UpduinoV2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from typing                             import Dict, Literal

from torii.build                        import (
	Connector, Resource, Pins, Clock, Attrs, Subsignal, PinsN,
	DiffPairs
)
from torii.build.run                    import BuildProducts
from torii.platform.vendor.lattice.ecp5 import ECP5Platform
from torii.platform.resources           import (
	LEDResources, ButtonResources, SwitchResources, UARTResource, SPIFlashResources
)

__all__ = (
	'ECP55GEVNPlatform',
)


class ECP55GEVNPlatform(ECP5Platform):
	device      = 'LFE5UM5G-85F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk12'
	default_rst = 'rst'

	def __init__(
		self, *, VCCIO1: Literal['3V3', '2V5'] = '2V5', VCCIO6: Literal['3V3', '2V5'] = '3V3', **kwargs
	) -> None:
		'''
		VCCIO1 is connected by default to 2.5 V via R100 (can be set to 3.3 V by disconnecting
		R100 and connecting R105)
		VCCIO6 is connected to 3.3 V by default via R99 (can be switched to 2.5 V with R104,
		see page 51 in the ECP5-5G-EVN datasheet)
		'''
		super().__init__(**kwargs)
		if VCCIO1 not in ('3V3', '2V5'):
			raise ValueError(f'VCCIO1 is expected to be \'3V3\' or \'2V5\' not {VCCIO1!r}')
		if VCCIO6 not in ('3V3', '2V5'):
			raise ValueError(f'VCCIO6 is expected to be \'3V3\' or \'2V5\' not {VCCIO6!r}')

		self._VCCIO1 = VCCIO1
		self._VCCIO6 = VCCIO6

	def _vccio_to_iostandard(self, vccio: Literal['3V3', '2V5']) -> str:
		if vccio == '2V5':
			return 'LVCMOS25'
		if vccio == '3V3':
			return 'LVCMOS33'

		raise ValueError(f'vccio is expected to be \'3V3\' or \'2V5\' not {vccio!r}')

	def bank1_iostandard(self) -> str:
		return self._vccio_to_iostandard(self._VCCIO1)

	def bank6_iostandard(self) -> str:
		return self._vccio_to_iostandard(self._VCCIO6)

	resources   = [
		Resource('rst', 0, PinsN('G2', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource(
			'clk12', 0, Pins('A10', dir = 'i'), Clock(12e6), Attrs(IO_TYPE = 'LVCMOS33')
		),

		# By default this CLK is not populated, see User Manual Section 4.
		Resource(
			'extclk', 0, Pins('B11', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')
		),

		*LEDResources(
			pins = 'A13 A12 B19 A18 B18 C17 A17 B17', invert = True,
			attrs = Attrs(IO_TYPE = bank1_iostandard)
		),
		*ButtonResources(
			pins = 'P4', invert = True, attrs = Attrs(IO_TYPE = bank6_iostandard)
		),
		*SwitchResources(
			pins = {1: 'J1', 2: 'H1', 3: 'K1'}, invert = True,
			attrs = Attrs(IO_TYPE = bank6_iostandard)
		),
		*SwitchResources(
			pins = {4: 'E15', 5: 'D16', 6: 'B16', 7: 'C16', 8: 'A16'}, invert = True,
			attrs = Attrs(IO_TYPE = bank1_iostandard)
		),

		# In order to use the UART as a UART you need to swap two resistor jumpers,
		# and potentially reconfigure the onboard FTDI chip, see section 6.2 in the
		# User Manual for hardware instructions, and
		# https://github.com/trabucayre/fixFT2232_ecp5evn to reconfigure the FTDI.
		UARTResource(0,
			rx = 'P2', tx = 'P3',
			attrs = Attrs(IO_TYPE = bank6_iostandard, PULLMODE = 'UP')
		),

		*SPIFlashResources(0,
			cs_n = 'R2', clk = 'U3', cipo = 'V2', copi = 'W2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),

		Resource('serdes', 0,
			Subsignal('tx', DiffPairs('W4', 'W5', dir = 'o')),
			Subsignal('rx', DiffPairs('Y5', 'Y6', dir = 'i')),
		),
		Resource('serdes', 1,
			Subsignal('tx', DiffPairs('W8', 'W9', dir = 'o')),
			Subsignal('rx', DiffPairs('Y7', 'Y8', dir = 'i')),
		),
		Resource('serdes', 2,
			Subsignal('tx', DiffPairs('W13', 'W14', dir = 'o')),
			Subsignal('rx', DiffPairs('Y14', 'Y15', dir = 'i')),
		),
		Resource('serdes', 3,
			Subsignal('tx', DiffPairs('W17', 'W18', dir = 'o')),
			Subsignal('rx', DiffPairs('Y16', 'Y17', dir = 'i')),
		),

		Resource('serdes_clk', 0, DiffPairs('Y11', 'Y12', dir = 'i')),
		Resource('serdes_clk', 1, DiffPairs('Y19', 'W20', dir = 'i')), # 200 MHz

		# TODO: add other resources
	]
	connectors  = [
		# Expansion connectors
		Connector('J', 39,
			'- - - D15 B15 C15 B13 B20 D11 E11 B12 C12 D12 E12 C13 D13  '
			'E13 A14 A9 B10 - - - - - - - - E7 - A11 - A19 - - - - - - -'
		),
		Connector('J', 40,
			'K2 - A15 F1 H2 G1 J4 J5 J3 K3 L4 L5 M4 N5 N4 P5 N3 M3 - - K5'
			'- M5 - L3 - N2 M1 L2 - L1 N1 C14 - P1 E14 D14 - K4 -  - - - '
		),

		# Arduino
		Connector('J', 6, 'K16 J16 H17 J17 H18 H16 - G18 G16 F17'),
		Connector('J', 3, 'F19 F20 E20 E19 D19 D20 C20 K17'),
		Connector('J', 7, 'C18 - D17 - - - - -'),
		Connector('J', 4, 'F18 E17 E18 D18 F16 E16'),
		# Raspberry Pi
		Connector('JP', 8,
			'- - T17 - U16 - U17 P18 - N20 N19 T16 M18 - N17 P17 - M17 U20 - '
			'T19 N18 R20 U19 - R18 L18 L17 U18 - T18 T20 P20 - R17 P19 N16 P16 - R16'
		),
		# GPIO
		Connector('J', 5, '- - H20 G19 - - K18 J18 - - K19 J19 - - K20 J20 - - G20 - - -'), # Contains 4 differential pairs
		Connector('J', 8, '- - L19 M19 L20 M20 L16 -'),
		Connector('J', 32,
			'- - - - A5 A4 - - C5 B5 - - B4 C4 - - B3 A3 - - D5 E4 - - D3 C3 - - E3'
			'F4 - - F5 E5 - - B1 A2 - -'
		), # Contains 9 differential pairs
		Connector('J', 33,
			'- - - - C2 B2 - - D1 C1 - - E1 D2 - - G5 H4 - - H3 H5 - - F3 G3 - - E2 F2 - -'
		), # Contains 7 differential pairs
		# Mic
		Connector('J', 30, '- B6 D9 C9 E9 D10 A6 E10 - -'),
		# PMOD
		Connector('J', 31, 'C6 C7 E8 D8 - - C8 B8 A7 A8 - -'),
		# JTAG
		Connector('J', 1, '- V4 R5 - - U5 - T5'),
		# Parallel configuration
		Connector('J', 38, 'W3 R2 T3 Y3 R1 V3 T1 V2 U1 W2 V1 T2 W1 U2 Y2 R2 U3 R3 - -'), # Connect pin 2 / R2 with jumper when needed
	]

	@property
	def file_templates(self) -> Dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': r'''
			interface ftdi
			ftdi_device_desc 'Lattice ECP5 Evaluation Board'
			ftdi_vid_pid 0x0403 0x6010
			ftdi_channel 0
			ftdi_layout_init 0xfff8 0xfffb
			reset_config none
			adapter_khz 25000

			jtag newtap ecp5 tap -irlen 8 -expected-id 0x81113043
			'''
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.svf') as files:
			(config_filename, vector_filename) = files
			check_call([
				openocd,
				'-f', config_filename,
				'-c', f'transport select jtag; init; svf -quiet {vector_filename}; exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ECP55GEVNPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Resource, Clock, Attrs, DiffPairs
from torii.build.run              import BuildProducts
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import LEDResources

__all__ = (
	'KCU105Platform',
)


class KCU105Platform(XilinxPlatform):
	device      = 'xcku040'
	package     = 'ffva1156'
	speed       = '2-e'
	default_clk = 'clk125'

	resources   = [
		Resource(
			'clk125', 0, DiffPairs('G10', 'F10', dir = 'i'), Clock(125e6), Attrs(IOSTANDARD = 'LVDS')
		),

		*LEDResources(
			pins = 'AP8 H23 P20 P21 N22 M22 R23 P23',
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
	]
	connectors  = []

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([openocd,
				'-c', f'source [find board/kcu105.cfg]; init; pld load 0 {bitstream_filename}; exit'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	KCU105Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                  import Resource, Pins, Clock, Attrs, Connector
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import SPIFlashResources, SRAMResource

__all__ = (
	'SK_XC6SLX9Platform',
)


class SK_XC6SLX9Platform(XilinxPlatform):
	device      = 'xc6slx9'
	package     = 'tqg144'
	speed       = '2'
	default_clk = 'clk50'

	resources   = [
		Resource(
			'clk50', 0, Pins('P134', dir = 'i'), Clock(50e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'P38', clk = 'P70', copi = 'P64', cipo = '65',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		SRAMResource(0,
			cs_n = 'P97', oe_n = 'P45', we_n = 'P51',
			a = 'P39 P40 P41 P43 P44 P55 P56 P57 P58 P59 P82 P81 P80 P79 P78 P66 P62 P61 P60',
			d = 'P46 P47 P48 P50 P75 P74 P69 P67',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]
	connectors  = [
		Connector('x', 7,
			'-    -    P34  -    P33  P32  P30  P29  P27  P26  '
			'P24  P23  P22  P21  P17  P16  P15  P14  P12  P11  '
			'P10  P9   P8   P7   P6   P5   P2   P1   P143 P144 '
			'P141 P142 P139 P140 P137 P138 P132 P133 P127 P131 '
		),
		Connector('x', 9,
			'-    -    P93  -    P92  P88  P87  P85  P84  P83  '
			'P74  P75  P78  P79  P81  P80  P69  P82  P66  P67  '
			'P61  P62  P59  P60  P58  P57  P55  P56  P50  P51  '
			'P47  P48  P44  P46  P45  P43  P40  P41  P35  P39  '
		),
		Connector('x', 8,
			'-    -    P126 -    P123 P124 P120 P121 P118 P119 '
			'P116 P117 P114 P115 P111 P112 P104 P105 P101 P102 '
			'P99  P100 P97  P98  P94  P95  -    -    -    -    '
			'-    -    -    -    -    -    -    -    -    -    '
		),
	]

	# This board doesn't have an integrated programmer.



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                 import Connector, Resource, Pins, Clock, Attrs
from torii.build.run             import BuildProducts
from torii.platform.vendor.altera import AlteraPlatform
from torii.platform.resources    import (
	LEDResources, ButtonResources, SwitchResources, Display7SegResource
)

__all__ = (
	'DE1SoCPlatform',
)


class DE1SoCPlatform(AlteraPlatform):
	device      = '5CSEMA5' # Cyclone V 85K LEs
	package     = 'F31'     # FBGA-896
	speed       = 'C6'
	default_clk = 'clk50'

	resources   = [
		Resource(
			'clk50', 0, Pins('AF14', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('AA16', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 2, Pins('Y26', dir = 'i'),  Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 3, Pins('K14', dir = 'i'),  Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'V16 W16 V17 V18 W17 W19 Y19 W20 W21 Y21',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'AA14 AA15 W15 Y16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'AB12 AC12 AF9 AF10 AD11 AD12 AE11 AC9 AD10 AE12',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(0,
			a = 'AE26', b = 'AE27', c = 'AE28', d = 'AG27', e = 'AF28',
			f = 'AG28', g = 'AH28', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(1,
			a = 'AJ29', b = 'AH29', c = 'AH30', d = 'AG30', e = 'AF29',
			f = 'AF30', g = 'AD27', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(2,
			a = 'AB23', b = 'AE29', c = 'AD29', d = 'AC28', e = 'AD30',
			f = 'AC29', g = 'AC30', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(3,
			a = 'AD26', b = 'AC27', c = 'AD25', d = 'AC25', e = 'AB28',
			f = 'AB25', g = 'AB22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(4,
			a = 'AA24', b = 'Y23', c = 'Y24', d = 'W22', e = 'W24',
			f = 'V23', g = 'W25', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(5,
			a = 'V25', b = 'AA28', c = 'Y27', d = 'AB27', e = 'AB26',
			f = 'AA26', g = 'AA25', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
	]

	connectors  = [
		# Located on the right hand side of the board
		Connector('gpio', 0,
			'AC18 Y17  AD17 Y18  AK16 AK18 AK19 AJ19 AJ17 AJ16 '
			' -    -   AH18 AH17 AG16 AE16 AF16 AG17 AA18 AA19 '
			'AE17 AC20 AH19 AJ20 AH20 AK21 AD19 AD20  -    -   '
			'AE18 AE19 AF20 AF21 AF19 AG21 AF18 AG20 AG18 AJ21 '
		),

		Connector('gpio', 1,
			'AB17 AA21 AB21 AC23 AD24 AE23 AE24 AF25 AF26 AG25 '
			'-    -    AG26 AH24 AH27 AJ27 AK29 AK28 AK27 AJ26 '
			'AK26 AH25 AJ25 AJ24 AK24 AG23 AK23 AH23  -    -   '
			'AK22 AJ22 AH22 AG22 AF24 AF23 AE22 AD21 AA20 AC22 '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			# The @2 selects the second device in the JTAG chain, because this chip
			# puts the ARM cores first.
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename + '@2'
			])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	DE1SoCPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Connector, Resource, Pins, Clock, Attrs
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, UARTResource, IrDAResource, SPIFlashResources
)

__all__ = (
	'ICEStickPlatform',
)


class ICEStickPlatform(ICE40Platform):
	device      = 'iCE40HX1K'
	package     = 'TQ144'
	default_clk = 'clk12'

	resources   = [
		Resource(
			'clk12', 0, Pins('21', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = '99 98 97 96 95', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		UARTResource(0,
			rx = '9', tx = '8', rts = '7', cts = '4', dtr = '3', dsr = '2', dcd = '1',
			attrs = Attrs(IO_STANDARD = 'SB_LVTTL', PULLUP = 1),
			role = 'dce'
		),

		IrDAResource(0,
			rx = '106', tx = '105', sd = '107',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*SPIFlashResources(0,
			cs_n = '71', clk = '70', copi = '67', cipo = '68',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors  = [
		Connector('pmod', 0, '78 79 80 81 - - 87 88 90 91 - -'),  # J2

		Connector('j', 1, '- - 112 113 114 115 116 117 118 119'), # J1
		Connector('j', 3, '- -  62  61  60  56  48  47  45  44'), # J3
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	ICEStickPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import (
	Connector, Resource, Pins, Clock, Attrs, PinsN
)
from torii.build.run                     import BuildProducts
from torii.platform.vendor.lattice.ice40 import ICE40Platform
from torii.platform.resources            import (
	LEDResources, ButtonResources, UARTResource, SPIFlashResources
)

__all__ = (
	'ICEBreakerPlatform',
)


class ICEBreakerPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),

		*LEDResources(pins = '11 37', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		# Semantic aliases
		Resource('led_r', 0, PinsN('11', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, PinsN('37', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),

		*ButtonResources(pins = '10', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),

		UARTResource(0,
			rx = '6', tx = '9',
			attrs = Attrs(IO_STANDARD = 'SB_LVTTL', PULLUP = 1)
		),

		*SPIFlashResources(0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '12', hold_n = '13',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors = [
		Connector('pmod', 0, ' 4  2 47 45 - -  3 48 46 44 - -'), # PMOD1A
		Connector('pmod', 1, '43 38 34 31 - - 42 36 32 28 - -'), # PMOD1B
		Connector('pmod', 2, '27 25 21 19 - - 26 23 20 18 - -'), # PMOD2
	]
	# The attached LED/button section can be either used standalone or as a PMOD.
	# Attach to platform using:
	# p.add_resources(p.break_off_pmod)
	# pmod_btn = plat.request('user_btn')
	break_off_pmod = [
		*LEDResources(
			pins = {
				2: '7', 3: '1', 4: '2', 5: '8', 6: '3'
			}, conn = ('pmod', 2), attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		# Semantic aliases
		Resource(
			'led_r', 1, Pins('7', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 1, Pins('1', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 2, Pins('2', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 3, Pins('8', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 4, Pins('3', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),

		*ButtonResources(
			pins = {
				1: '9', 2: '4', 3: '10'
			}, conn = ('pmod', 2), attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ..test.blinky import Blinky
	p = ICEBreakerPlatform()
	p.add_resources(p.break_off_pmod)
	p.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from typing                             import List

from torii.build                        import (
	Connector, Resource, Pins, Clock, Attrs, PinsN,
	Subsignal, DiffPairs,
)
from torii.build.run                    import BuildProducts, BuildPlan
from torii.hdl.ir                       import Fragment
from torii.platform.vendor.lattice.ecp5 import ECP5Platform
from torii.platform.resources           import (
	LEDResources, ButtonResources, UARTResource, SDCardResources, SDRAMResource,
	SPIResource, DirectUSBResource
)

__all__ = (
	'ULX3S_12F_Platform',
	'ULX3S_25F_Platform',
	'ULX3S_45F_Platform',
	'ULX3S_85F_Platform',
)


class _ULX3SPlatform(ECP5Platform):
	package     = 'BG381'
	speed       = '6'
	default_clk = 'clk25'

	resources   = [
		Resource('clk25', 0, Pins('G2', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS33')),

		# Used to reload FPGA configuration.
		Resource('program', 0, PinsN('M4', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')),

		*LEDResources(pins = 'B2 C2 C1 D2 D1 E2 E1 H3',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')),
		*ButtonResources(pins = 'R1 T1 R18 V1 U1 H16',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')
		),
		*ButtonResources('switch', pins = 'E8 D8 D7 E7',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')
		),

		# Semantic aliases by button label.
		Resource('button_pwr',   0, PinsN('D6', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')),
		Resource('button_fire',  0, Pins('R1',  dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_fire',  1, Pins('T1',  dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_up'                ,    0, Pins('R18', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_down',  0, Pins('V1',  dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_left',  0, Pins('U1',  dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_right', 0, Pins('H16', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),

		# FTDI connection.
		UARTResource(0,
			rx = 'M1', tx = 'L4', rts = 'M3', dtr = 'N1', role = 'dce',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('uart_tx_enable', 0, Pins('L3', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),

		*SDCardResources(0,
			clk = 'J1', cmd = 'J3', dat0 = 'K2', dat1 = 'K1', dat2 = 'H2', dat3 = 'H1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEW = 'FAST')
		),

		# SPI Flash clock is accessed via USR_MCLK instance.
		Resource('spi_flash', 0,
			Subsignal('cs',   PinsN('R2', dir = 'o')),
			Subsignal('copi', Pins('W2', dir = 'o')),
			Subsignal('cipo', Pins('V2', dir = 'i')),
			Subsignal('hold', PinsN('W1', dir = 'o')),
			Subsignal('wp',   PinsN('Y2', dir = 'o')),
			Attrs(PULLMODE = 'NONE', DRIVE = '4', IO_TYPE = 'LVCMOS33')
		),

		SDRAMResource(0,
			clk = 'F19', cke = 'F20', cs_n = 'P20', we_n = 'T20', cas_n = 'T19', ras_n = 'R20', dqm = 'U19 E20',
			ba = 'P19 N20', a = 'M20 M19 L20 L19 K20 K19 K18 J20 J19 H20 N19 G20 G19',
			dq = 'J16 L18 M18 N18 P18 T18 T17 U20 E19 D20 D19 C20 E18 F18 J18 J17',
			attrs = Attrs(PULLMODE = 'NONE', DRIVE = '4', SLEWRATE = 'FAST', IO_TYPE = 'LVCMOS33')
		),

		# SPI bus for ADC.
		SPIResource('adc', 0, cs_n = 'R17', copi = 'R16', cipo = 'U16', clk = 'P17',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')),

		# TRRS audio jack
		Resource('audio', 0,
			Subsignal('l', Pins('E4 D3 C3 B3', dir = 'o')),
			Subsignal('r', Pins('A3 B5 D5 C5', dir = 'o')),
			Subsignal('ring2', Pins('H5 F2 F5 E5', dir = 'o')), # extra ring out for video adapters
		),

		# ESP-32 connections
		Resource('esp32', 0,
			Subsignal('en',     Pins('F1', dir = 'o'), Attrs(PULLMODE = 'UP')),
			Subsignal('tx',     Pins('K3', dir = 'o'), Attrs(PULLMODE = 'UP')),
			Subsignal('rx',     Pins('K4', dir = 'i'), Attrs(PULLMODE = 'UP')),
			Subsignal('gpio0',  Pins('L2'),          Attrs(PULLMODE = 'UP')),
			Subsignal('gpio5',  Pins('N4')),
			Subsignal('gpio16', Pins('L1'),          Attrs(PULLMODE = 'UP')),
			Subsignal('gpio17', Pins('N3'),          Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')
		),

		# PCB antenna, tuned to 433MHz
		Resource('ant', 0, Pins('G1', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),

		# Differential versions of our connector I/O.
		Resource('diff_gpio', 0, DiffPairs('B11', 'C11'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 1, DiffPairs('A10', 'A11'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 2, DiffPairs('A9', 'B10'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 3, DiffPairs('B9', 'C10'),  Attrs(IO_TYPE = 'LVCMOS33')),

		# HDMI (only TX, due to the top bank of ECP5 only supporting diff. outputs)
		Resource('hdmi', 0,
			Subsignal(
				'cec', Pins('A18', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			),
			Subsignal(
				'clk', DiffPairs('A17', 'B18', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'd',   DiffPairs('A16 A14 A12', 'B16 C14 A13', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'eth', DiffPairs('A19', 'B20', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'scl', Pins('E12', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			),
			Subsignal(
				'sda', Pins('B19', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			)
		),

		DirectUSBResource(0,
			d_p = 'D15', d_n = 'E15', pullup = 'B12',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		)
	]

	connectors = [
		Connector('gpio', 0, {
			'0+': 'B11',  '0-':  'C11', '1+':  'A10', '1-':  'A11',
			'2+': 'A9',   '2-':  'B10', '3+':  'B9',  '3-':  'C10',
			'4+': 'A7',   '4-':  'A8',  '5+':  'C8',  '5-':  'B8',
			'6+': 'C6',   '6-':  'C7',  '7+':  'A6',  '7-':  'B6',
			'8+': 'A4',   '8-':  'A5',  '9+':  'A2',  '9-':  'B1',
			'10+': 'C4',  '10-': 'B4',  '11+': 'F4',  '11-': 'E3',
			'12+': 'G3',  '12-': 'F3',  '13+': 'H4',  '13-': 'G5',
			'14+': 'U18', '14-': 'U17', '15+': 'N17', '15-': 'P16',
			'16+': 'N16', '16-': 'M17', '17+': 'L16', '17-': 'L17',
			'18+': 'H18', '18-': 'H17', '19+': 'F17', '19-': 'G18',
			'20+': 'D18', '20-': 'E17', '21+': 'C18', '21-': 'D17',
			'22+': 'B15', '22-': 'C15', '23+': 'B17', '23-': 'C17',
			'24+': 'C16', '24-': 'D16', '25+': 'D14', '25-': 'E14',
			'26+': 'B13', '26-': 'C13', '27+': 'D13', '27-': 'E13',
		})
	]

	@property
	def required_tools(self) -> List[str]:
		return super().required_tools + [
			'openFPGALoader'
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([tool, '-b', 'ulx3s', '-m', bitstream_filename])


class ULX3S_12F_Platform(_ULX3SPlatform):
	device = 'LFE5U-12F'


class ULX3S_25F_Platform(_ULX3SPlatform):
	device = 'LFE5U-25F'


class ULX3S_45F_Platform(_ULX3SPlatform):
	device = 'LFE5U-45F'


class ULX3S_85F_Platform(_ULX3SPlatform):
	device = 'LFE5U-85F'


if __name__ == '__main__':
	from argparse      import ArgumentParser
	from ..test.blinky import Blinky

	variants = {
		'12F': ULX3S_12F_Platform,
		'25F': ULX3S_25F_Platform,
		'45F': ULX3S_45F_Platform,
		'85F': ULX3S_85F_Platform
	}

	# Figure out which FPGA variant we want to target...
	parser = ArgumentParser()
	parser.add_argument('variant', choices = variants.keys())
	args = parser.parse_args()

	# ... and run Blinky on it.
	platform = variants[args.variant]
	platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from typing                       import Literal

from torii.build                  import (
	Resource, Pins, Clock, Attrs, Subsignal, Connector, PinsN,
	DiffPairs
)
from torii.build.run              import BuildProducts
from torii.platform.vendor.xilinx import XilinxPlatform
from torii.platform.resources     import (
	UARTResource, PS2Resource, SPIFlashResources
)

__all__ = (
	'AtlysPlatform',
)


class AtlysPlatform(XilinxPlatform):
	'''Platform file for Digilent Atlys Spartan 6 board.
	https://reference.digilentinc.com/reference/programmable-logic/atlys/start'''

	device  = 'xc6slx45'
	package = 'csg324'
	speed   = '3'

	def __init__(self, *, JP12: Literal['2V5', '3V3'] = '2V5', **kwargs) -> None:
		super().__init__(**kwargs)

		if JP12 not in ('2V5', '3V3'):
			raise ValueError(f'JP12 must be either \'2V5\' or \'3V3\' not {JP12!r}')
		self._JP12 = JP12

	def bank2_iostandard(self) -> str:
		return 'LVCMOS25' if self._JP12 == '2V5' else 'LVCMOS33'

	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource('rst',    0, PinsN('T15', dir = 'i'), Attrs(IOSTANDARD = bank2_iostandard)), # RESET
		Resource(
			'clk100', 0, Pins('L15',  dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')
		), # GCLK

		Resource('led',    0, Pins('U18',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD0
		Resource('led',    1, Pins('M14',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD1
		Resource('led',    2, Pins('N14',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD2
		Resource('led',    3, Pins('L14',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD3
		Resource('led',    4, Pins('M13',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD4
		Resource('led',    5, Pins('D4',   dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD5
		Resource('led',    6, Pins('P16',  dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),       # LD6
		Resource('led',    7, Pins('N12',  dir = 'o'), Attrs(IOSTANDARD = bank2_iostandard)), # LD7

		Resource('button', 0, Pins('N4',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # BTNU
		Resource('button', 1, Pins('P4',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # BTNL
		Resource('button', 2, Pins('P3',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # BTND
		Resource('button', 3, Pins('F6',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # BTNR
		Resource('button', 4, Pins('F5',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # BTNC

		Resource('switch', 0, Pins('A10',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),       # SW0
		Resource('switch', 1, Pins('D14',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),       # SW1
		Resource('switch', 2, Pins('C14',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),       # SW2
		Resource('switch', 3, Pins('P15',  dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),       # SW3
		Resource('switch', 4, Pins('P12',  dir = 'i'), Attrs(IOSTANDARD = bank2_iostandard)), # SW4
		Resource('switch', 5, Pins('R5',   dir = 'i'), Attrs(IOSTANDARD = bank2_iostandard)), # SW5
		Resource('switch', 6, Pins('T5',   dir = 'i'), Attrs(IOSTANDARD = bank2_iostandard)), # SW6
		Resource('switch', 7, Pins('E4',   dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')),       # SW7

		UARTResource(0, rx = 'A16', tx = 'B16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')), # J17/UART

		PS2Resource(0, # PS/2 keyboard interface converted from J13 'HOST' USB connector
			clk = 'P17', dat = 'N15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		PS2Resource(1, # PS/2 mouse interface converted from J13 'HOST' USB connector
			clk = 'N18', dat = 'P18', attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),

		*SPIFlashResources(0,
			cs_n = 'AE14', clk = 'AH18', copi = 'AF14', cipo = 'AF20', wp_n = 'AG21', hold_n = 'AG17',
			attrs = Attrs(IOSTANDARD = 'LVCMOS25', SLEW = 'FAST'),
		),

		Resource('ddr2', 0,
			Subsignal('clk',    DiffPairs('G3', 'G1', dir = 'o'),
								Attrs(IOSTANDARD = 'DIFF_SSTL18_II', IN_TERM = 'NONE')),
			Subsignal('clk_en', Pins('H7', dir = 'o')),
			Subsignal('we',     PinsN('E3', dir = 'o')),
			Subsignal('ras',    PinsN('L5', dir = 'o')),
			Subsignal('cas',    PinsN('K5', dir = 'o')),
			Subsignal('a',      Pins('J7 J6 H5 L7 F3 H4 H3 H6 D2 D1 F4 D3 G6', dir = 'o')),
			Subsignal('ba',     Pins('F2 F1 E1', dir = 'o')),
			Subsignal('dqs',    DiffPairs('P2 L4', 'P1 L3', dir = 'o'),
								Attrs(IOSTANDARD = 'DIFF_SSTL18_II')),
			Subsignal('dq',     Pins('L2 L1 K2 K1 H2 H1 J3 J1 M3 M1 N2 N1 T2 T1 U2 U1', dir = 'io')),
			Subsignal('dm',     Pins('K4 K3', dir = 'o')),
			Subsignal('odt',    Pins('K6', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL18_II', SLEW = 'FAST'),
		),

		Resource('eth_gmii', 0,
			Subsignal('rst',     PinsN('G13', dir = 'o')),
			Subsignal('int',     PinsN('L16', dir = 'o')),
			Subsignal('mdio',    Pins('N17', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('gtx_clk', Pins('L12', dir = 'o')),
			Subsignal('tx_clk',  Pins('K16', dir = 'i')),
			Subsignal('tx_en',   Pins('H15', dir = 'o')),
			Subsignal('tx_er',   Pins('G18', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 J13 G14 H12 K12', dir = 'o')),
			Subsignal('rx_clk',  Pins('K15', dir = 'i')),
			Subsignal('rx_dv',   Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_er',   Pins('F18', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F14 E18 D16 D17', dir = 'i')),
			Subsignal('col',     Pins('C17', dir = 'i')),
			Subsignal('crs',     Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('eth_rgmii', 0,
			Subsignal('rst',     PinsN('G13', dir = 'o')),
			Subsignal('int',     PinsN('L16', dir = 'o')),
			Subsignal('mdio',    Pins('N17', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk',  Pins('L12', dir = 'o')),
			Subsignal('tx_ctl',  Pins('H15', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13', dir = 'o')),
			Subsignal('rx_clk',  Pins('K15', dir = 'i')),
			Subsignal('rx_ctl',  Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('eth_mii', 0,
			Subsignal('rst',     PinsN('G13', dir = 'o')),
			Subsignal('int',     PinsN('L16', dir = 'o')),
			Subsignal('mdio',    Pins('N17', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk',  Pins('K16', dir = 'i')),
			Subsignal('tx_en',   Pins('H15', dir = 'o')),
			Subsignal('tx_er',   Pins('G18', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13', dir = 'o')),
			Subsignal('rx_clk',  Pins('K15', dir = 'i')),
			Subsignal('rx_dv',   Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_er',   Pins('F18', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15', dir = 'i')),
			Subsignal('col',     Pins('C17', dir = 'i')),
			Subsignal('crs',     Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# Device does not support RMII
		Resource('eth_tbi', 0,
			Subsignal('rst',     PinsN('G13', dir = 'o')),
			Subsignal('int',     PinsN('L16', dir = 'o')),
			Subsignal('mdio',    Pins('N17', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk',  Pins('L12', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 J13 G14 H12 K12 H15 G18', dir = 'o')),
			Subsignal('rx_clk',  Pins('K15 L12', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F14 E18 D16 D17 F17 F18', dir = 'i')),
			Subsignal('lpbk',    Pins('C17', dir = 'o'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('comma',   Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('eth_rtbi', 0,
			Subsignal('rst',     PinsN('G13', dir = 'o')),
			Subsignal('int',     PinsN('L16', dir = 'o')),
			Subsignal('mdio',    Pins('N17', dir = 'io')),
			Subsignal('mdc',     Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk',  Pins('L12', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 H15', dir = 'o')),
			Subsignal('rx_clk',  Pins('K15', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource('hdmi', 0, # J1, input only due to on board buffer, HDMI A connector
			Subsignal('scl',     Pins('C13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda',     Pins('A13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk',     DiffPairs('D11', 'C11', dir = 'i')),
			Subsignal('d',       DiffPairs('G9 B11 B12', 'F9 A11 A12', dir = 'i')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		Resource('hdmi', 1, # J2, output only due to on board buffer, HDMI A connector
			Subsignal('scl',     Pins('D9'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda',     Pins('C9'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk',     DiffPairs('B6', 'A6', dir = 'o')),
			Subsignal('d',       DiffPairs('D8 C7 B8', 'C8 A7 A8', dir = 'o')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		Resource('hdmi', 2, # J3, input only due to on board buffer, HDMI A connector
			Subsignal('scl',     Pins('M16'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda',     Pins('M18'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk',     DiffPairs('H17', 'H18', dir = 'i')),
			Subsignal('d',       DiffPairs('K17 L17 J16', 'K18 L18 J18', dir = 'i')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		Resource('hdmi', 3, # JA, input/output as it is unbuffered, HDMI D connector
			Subsignal('scl',     Pins('C13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda',     Pins('A13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk',     DiffPairs('T9', 'V9')),
			Subsignal('d',       DiffPairs('R3 T4 N5', 'T3 V4 P6')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),

		Resource('ac97', 0,
			Subsignal('clk',     Pins('L13', dir = 'o')),
			Subsignal('sync',    Pins('U17', dir = 'o')),
			Subsignal('reset',   Pins('T17', dir = 'o')),
			Subsignal('sdo',     Pins('N18', dir = 'o')),
			Subsignal('sdi',     Pins('T18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]
	connectors  = [
		Connector('pmod', 0, 'T3 R3 P6 N5 - - V9 T9 V4 T4 - -'), # JB

		Connector('vhdci', 0, # JC
			'U16 - U15 U13 - M11 R11 - T12 N10 - M10 U11 - R10 - - - - U10 - R8  M8  - U8  U7  - N7  T6  - R7  N6  - U5 '
			'V16 - V15 V13 - N11 T11 - V12 P11 - N9  V11 - T10 - - - - V10 - T8  N8  - V8  V7  - P8  V6  - T7  P7  - V5 '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from textwrap   import dedent
		from subprocess import run

		from torii.tools import require_tool

		impact = require_tool('impact')

		with products.extract(f'{name}.bit') as bitstream:
			cmd = dedent(f'''
				setMode -bscan
				setCable -port auto
				addDevice -p 1 -file '{bitstream}'
				program -p 1
				exit
			''').encode('utf-8')
			run([impact, '-batch'], input = cmd, check = True)


if __name__ == '__main__':
	from ..test.blinky import Blinky
	AtlysPlatform().build(Blinky(), do_program = True)
